﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Security.Cryptography
Imports System.Text
Imports EFCachingProvider.Caching
Imports Microsoft.ApplicationServer.Caching

''' <summary>
''' Implementation of <see cref="ICache"/> which works with Microsoft Distributed Cache codename "AppFabric" CTP3.
''' </summary>
Public Class AppFabricCache
    Implements ICache
    Private cache As DataCache

    ''' <summary>
    ''' Initializes a new instance of the AppFabricCache class.
    ''' </summary>
    ''' <param name="cache">The cache to be used.</param>
    Public Sub New(ByVal cache As DataCache)
        Me.cache = cache
    End Sub

    ''' <summary>
    ''' Tries to the get cached entry by key.
    ''' </summary>
    ''' <param name="key">The cache key.</param>
    ''' <param name="value">The retrieved value.</param>
    ''' <returns>
    ''' A value of <c>true</c> if entry was found in the cache, <c>false</c> otherwise.
    ''' </returns>
    Public Function GetItem(ByVal key As String, <System.Runtime.InteropServices.Out()> ByRef value As Object) As Boolean
        key = GetCacheKey(key)
        value = Me.cache.Get(key)

        Return value IsNot Nothing
    End Function

    ''' <summary>
    ''' Adds the specified entry to the cache.
    ''' </summary>
    ''' <param name="key">The entry key.</param>
    ''' <param name="value">The entry value.</param>
    ''' <param name="dependentEntitySets">The list of dependent entity sets.</param>
    ''' <param name="slidingExpiration">The sliding expiration - IGNORED.</param>
    ''' <param name="absoluteExpiration">The absolute expiration.</param>
    Public Sub PutItem(ByVal key As String, ByVal value As Object, ByVal dependentEntitySets As IEnumerable(Of String), ByVal slidingExpiration As TimeSpan, ByVal absoluteExpiration As Date)
        key = GetCacheKey(key)
        Me.cache.Put(key, value, absoluteExpiration.Subtract(Date.Now), dependentEntitySets.Select(Function(c) New DataCacheTag(c)).ToList())

        For Each dep In dependentEntitySets
            Me.CreateRegionIfNeeded(dep)
            Me.cache.Put(key, " ", dep)
        Next dep

    End Sub

    ''' <summary>
    ''' Invalidates all cache entries which are dependent on any of the specified entity sets.
    ''' </summary>
    ''' <param name="entitySets">The entity sets.</param>
    Public Sub InvalidateSets(ByVal entitySets As IEnumerable(Of String))
        ' Go through the list of objects in each of the set. 
        For Each dep In entitySets
            For Each Val() In Me.cache.GetObjectsInRegion(dep)
                Me.cache.Remove(Val.Key)
            Next Val()
        Next dep
    End Sub

    ''' <summary>
    ''' Invalidates cache entry with a given key.
    ''' </summary>
    ''' <param name="key">The cache key.</param>
    Public Sub InvalidateItem(ByVal key As String)
        key = GetCacheKey(key)

        Dim item As DataCacheItem = Me.cache.GetCacheItem(key)
        Me.cache.Remove(key)

        ' Remove the keys from the region..
        For Each tag In item.Tags
            Me.cache.Remove(key, tag.ToString())
        Next tag
    End Sub

    ''' <summary>
    ''' Hashes the query to produce cache key..
    ''' </summary>
    ''' <param name="query">The query.</param>
    ''' <returns>Hashed query which becomes a cache key.</returns>
    Private Shared Function GetCacheKey(ByVal query As String) As String
        Dim bytes() As Byte = Encoding.UTF8.GetBytes(query)
        Dim hashString As String = Convert.ToBase64String(MD5.Create().ComputeHash(bytes))
        Return hashString
    End Function

    ''' <summary>
    ''' Creates the region if needed.
    ''' </summary>
    ''' <param name="regionName">Name of the region.</param>
    Private Sub CreateRegionIfNeeded(ByVal regionName As String)
        Try
            'old code removed  > , false)
            Me.cache.CreateRegion(regionName)
        Catch de As DataCacheException
            If de.ErrorCode <> DataCacheErrorCode.RegionAlreadyExists Then
                Throw
            End If
        End Try
    End Sub
End Class