﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Data.Metadata.Edm

Namespace Caching
    ''' <summary>
    ''' Custom caching policy on per-table basis.
    ''' </summary>
    Public Class CustomCachingPolicy
        Inherits CachingPolicy
        ''' <summary>
        ''' Initializes a new instance of the CustomCachingPolicy class.
        ''' </summary>
        Public Sub New()
            Me.MinCacheableRows = 0
            Me.MaxCacheableRows = 1000
            Me.CacheableTables = New HashSet(Of String)(StringComparer.OrdinalIgnoreCase)
            Me.NonCacheableTables = New HashSet(Of String)(StringComparer.OrdinalIgnoreCase)
        End Sub

        ''' <summary>
        ''' Gets or sets the minimal number of cacheable rows.
        ''' </summary>
        ''' <value>Minimal number of cacheable rows.</value>
        Public Property MinCacheableRows() As Integer

        ''' <summary>
        ''' Gets or sets the maximum number of cacheable rows.
        ''' </summary>
        ''' <value>Maximum number of cacheable rows.</value>
        Public Property MaxCacheableRows() As Integer

        ''' <summary>
        ''' Gets the collection of cacheable tables.
        ''' </summary>
        '''The cacheable tables.
        Private _CacheableTables As ICollection(Of String)
        Public Property CacheableTables() As ICollection(Of String)
            Get
                Return _CacheableTables
            End Get
            Private Set(ByVal value As ICollection(Of String))
                _CacheableTables = value
            End Set
        End Property

        ''' <summary>
        ''' Gets the collection of non cacheable tables.
        ''' </summary>
        '''The non cacheable tables.
        Private _NonCacheableTables As ICollection(Of String)
        Public Property NonCacheableTables() As ICollection(Of String)
            Get
                Return _NonCacheableTables
            End Get
            Private Set(ByVal value As ICollection(Of String))
                _NonCacheableTables = value
            End Set
        End Property

        ''' <summary>
        ''' Determines whether the specified command definition can be cached.
        ''' </summary>
        ''' <param name="commandDefinition">The command definition.</param>
        ''' <returns>
        ''' A value of <c>true</c> if the specified command definition can be cached; otherwise, <c>false</c>.
        ''' </returns>
        Protected Friend Overrides Function CanBeCached(ByVal commandDefinition As EFCachingCommandDefinition) As Boolean
            If commandDefinition Is Nothing Then
                Throw New ArgumentNullException("commandDefinition")
            End If

            ' check if we have any non-cacheable tables in the query
            If Me.NonCacheableTables.Count > 0 Then
                For Each esb In commandDefinition.AffectedEntitySets
                    If Me.NonCacheableTables.Contains(esb.Name) Then
                        Return False
                    End If
                Next esb
            End If

            ' by default everything is cacheable
            If Me.CacheableTables.Count = 0 Then
                Return True
            Else
                ' unless the user has explicitly specified which tables to cache
                For Each esb In commandDefinition.AffectedEntitySets
                    If Not Me.CacheableTables.Contains(esb.Name) Then
                        Return False
                    End If
                Next esb

                Return True
            End If
        End Function

        ''' <summary>
        ''' Gets the minimum and maximum number cacheable rows for a given command definition.
        ''' </summary>
        ''' <param name="cachingCommandDefinition">The command definition.</param>
        ''' <param name="minCacheableRows">The minimum number of cacheable rows.</param>
        ''' <param name="maxCacheableRows">The maximum number of cacheable rows.</param>
        Protected Friend Overrides Sub GetCacheableRows(ByVal cachingCommandDefinition As EFCachingCommandDefinition, <System.Runtime.InteropServices.Out()> ByRef minCacheableRows As Integer, <System.Runtime.InteropServices.Out()> ByRef maxCacheableRows As Integer)
            minCacheableRows = Me.MinCacheableRows
            maxCacheableRows = Me.MaxCacheableRows
        End Sub
    End Class
End Namespace