﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Data
Imports System.Data.Common
Imports System.Data.Metadata.Edm
Imports System.Globalization
Imports System.Text
Imports System.Threading
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbCommand"/> wrappr which implements query caching.
''' </summary>
Public NotInheritable Class EFCachingCommand
    Inherits DbCommandWrapper

    Private Shared _cacheableCommands As Integer

    Private Shared _nonCacheableCommands As Integer

    Private Shared _cacheHits As Integer

    Private Shared _cacheMisses As Integer

    Private Shared _cacheAdds As Integer

    Private _transaction As EFCachingTransaction

    ''' <summary>
    ''' Initializes a new instance of the EFCachingCommand class.
    ''' </summary>
    ''' <param name="wrappedCommand">The wrapped command.</param>
    ''' <param name="commandDefinition">The command definition.</param>
    Public Sub New(ByVal wrappedCommand As DbCommand, ByVal commandDefinition As DbCommandDefinitionWrapper)
        MyBase.New(wrappedCommand, commandDefinition)
    End Sub

    ''' <summary>
    ''' Gets the number of cacheable commands.
    ''' </summary>
    ''' <value>The cacheable commands.</value>
    Public Shared ReadOnly Property CacheableCommands() As Integer
        Get
            Return _cacheableCommands
        End Get
    End Property

    ''' <summary>
    ''' Gets the number of non-cacheable commands.
    ''' </summary>
    ''' <value>The non cacheable commands.</value>
    Public Shared ReadOnly Property NonCacheableCommands() As Integer
        Get
            Return _nonCacheableCommands
        End Get
    End Property

    ''' <summary>
    ''' Gets the number of cache hits.
    ''' </summary>
    ''' <value>The cache hits.</value>
    Public Shared ReadOnly Property CacheHits() As Integer
        Get
            Return _cacheHits
        End Get
    End Property

    ''' <summary>
    ''' Gets the total number of cache misses.
    ''' </summary>
    Public Shared ReadOnly Property CacheMisses() As Integer
        Get
            Return _cacheMisses
        End Get
    End Property

    ''' <summary>
    ''' Gets the total number of cache adds.
    ''' </summary>
    ''' <value>The number of cache adds.</value>
    Public Shared ReadOnly Property CacheAdds() As Integer
        Get
            Return _cacheAdds
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the <see cref="P:System.Data.Common.DbCommand.DbTransaction"/> within which this <see cref="T:System.Data.Common.DbCommand"/> object executes.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The transaction within which a Command object of a .NET Framework data provider executes. The default value is a null reference (Nothing in Visual Basic).
    ''' </returns>
    Protected Overrides Property DbTransaction() As DbTransaction
        Get
            Return Me.WrappedCommand.Transaction
        End Get

        Set(ByVal value As DbTransaction)
            Me._transaction = CType(value, EFCachingTransaction)
            If Me._transaction IsNot Nothing Then
                WrappedCommand.Transaction = Me._transaction.WrappedTransaction
            Else
                WrappedCommand.Transaction = Nothing
            End If
        End Set
    End Property

    ''' <summary>
    ''' Gets <see cref="EFCachingConnection"/> used by this <see cref="T:System.Data.Common.DbCommand"/>.
    ''' </summary>
    ''' <returns>
    ''' The connection to the data source.
    ''' </returns>
    Private Shadows ReadOnly Property Connection() As EFCachingConnection
        Get
            Return CType(MyBase.Connection, EFCachingConnection)
        End Get
    End Property

    Private Shadows ReadOnly Property Definition() As EFCachingCommandDefinition
        Get
            Return CType(MyBase.Definition, EFCachingCommandDefinition)
        End Get
    End Property

    ''' <summary>
    ''' Executes a SQL statement against a connection object.
    ''' </summary>
    ''' <returns>The number of rows affected.</returns>
    Public Overrides Function ExecuteNonQuery() As Integer
        Me.UpdateAffectedEntitySets()
        Return WrappedCommand.ExecuteNonQuery()
    End Function

    ''' <summary>
    ''' Executes the query and returns the first column of the first row in the result set returned by the query. All other columns and rows are ignored.
    ''' </summary>
    ''' <returns>
    ''' The first column of the first row in the result set.
    ''' </returns>
    Public Overrides Function ExecuteScalar() As Object
        Me.UpdateAffectedEntitySets()
        Return WrappedCommand.ExecuteScalar()
    End Function

    ''' <summary>
    ''' Executes the command text against the connection.
    ''' </summary>
    ''' <param name="behavior">An instance of <see cref="T:System.Data.CommandBehavior"/>.</param>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbDataReader"/>.
    ''' </returns>
    Protected Overrides Function ExecuteDbDataReader(ByVal behavior As CommandBehavior) As DbDataReader
        Dim cache As ICache = Me.Connection.Cache
        If cache Is Nothing Then
            Interlocked.Increment(_nonCacheableCommands)
            Return WrappedCommand.ExecuteReader(behavior)
        End If

        Me.UpdateAffectedEntitySets()
        Dim cacheKey As String = Me.GetCacheKey()
        If cacheKey Is Nothing OrElse (Not Me.Definition.IsCacheable()) OrElse (Not Me.Connection.CachingPolicy.CanBeCached(Me.Definition)) Then
            ' non-cacheable
            Interlocked.Increment(_nonCacheableCommands)
            Return WrappedCommand.ExecuteReader(behavior)
        End If

        Dim value As New Object

        Interlocked.Increment(_cacheableCommands)
        If cache.GetItem(cacheKey, value) Then
            Interlocked.Increment(_cacheHits)

            ' got cache entry - create reader based on that
            Return New CachingDataReaderCacheReader(CType(value, DbQueryResults))
        Else
            Interlocked.Increment(_cacheMisses)
            Dim dependentEntitySets As New List(Of String)()
            For Each [set] As EntitySetBase In Me.Definition.AffectedEntitySets
                dependentEntitySets.Add([set].Name)
            Next [set]

            Dim minCacheableRows, maxCacheableRows As Integer

            Me.Connection.CachingPolicy.GetCacheableRows(Me.Definition, minCacheableRows, maxCacheableRows)

            Return New EFCachingDataReaderCacheWriter(Me.WrappedCommand.ExecuteReader(behavior), maxCacheableRows, Sub(entry As DbQueryResults)
                                                                                                                       If entry.Rows.Count >= minCacheableRows AndAlso entry.Rows.Count <= maxCacheableRows Then
                                                                                                                           Interlocked.Increment(_cacheAdds)
                                                                                                                           Dim absoluteExpiration As Date
                                                                                                                           Dim slidingExpiration As TimeSpan
                                                                                                                           Me.Connection.CachingPolicy.GetExpirationTimeout(Me.Definition, slidingExpiration, absoluteExpiration)
                                                                                                                           cache.PutItem(cacheKey, entry, dependentEntitySets, slidingExpiration, absoluteExpiration)
                                                                                                                       End If
                                                                                                                   End Sub)
        End If
    End Function

    Private Shared Function GetLiteralValue(ByVal value As Object) As String
        If TypeOf value Is String Then
            Return "'" & value.ToString().Replace("'", "''") & "'"
        Else
            Return Convert.ToString(value, CultureInfo.InvariantCulture)
        End If
    End Function

    Private Function GetCacheKey() As String
        Dim sb As New StringBuilder()
        sb.Append(CommandType)
        sb.Append("|")
        sb.Append(CommandText)

        For Each parameter In Parameters
            If parameter.Direction <> ParameterDirection.Input Then
                ' we don't cache queries with output parameters
                Return Nothing
            End If

            sb = sb.Replace("@" & parameter.ParameterName, GetLiteralValue(parameter.Value))
        Next parameter

#If HASH_COMMANDS Then
			Dim bytes() As Byte = Encoding.UTF8.GetBytes(sb.ToString())
			Dim hashString As String = Convert.ToBase64String(MD5.Create().ComputeHash(bytes))
			' Console.WriteLine("hashString: {0}", hashString)
			Return hashString
#Else
        Return sb.ToString()
#End If
    End Function

    Private Sub UpdateAffectedEntitySets()
        If Me._transaction IsNot Nothing Then
            If Me.Definition.IsModification Then
                Me._transaction.HasModifications = True
            End If

            For Each entitySet In Me.Definition.AffectedEntitySets
                Me._transaction.AddAffectedEntitySet(entitySet)
            Next entitySet
        End If
    End Sub
End Class