﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Data.Common
Imports System.Data.Common.CommandTrees
Imports System.Data.Metadata.Edm
Imports System.Linq
Imports EFProviderWrapperToolkit

''' <summary>
''' Represents a command definitio
''' </summary>
Public Class EFCachingCommandDefinition
    Inherits DbCommandDefinitionWrapper

    Private _affectedEntitySets As New List(Of EntitySetBase)()
    Private functionsUsed As New List(Of EdmFunction)()

    ''' <summary>
    ''' Initializes static members of the EFCachingCommandDefinition class.
    ''' </summary>
    Shared Sub New()
        NonCacheableFunctions = New HashSet(Of String)(StringComparer.OrdinalIgnoreCase) From {"Edm.CurrentDateTime", "Edm.CurrentUtcDateTime", "Edm.CurrentDateTimeOffsets", "Edm.NewGuid", "SqlServer.NEWID", "SqlServer.GETDATE", "SqlServer.GETUTCDATE", "SqlServer.SYSDATETIME", "SqlServer.SYSUTCDATETIME", "SqlServer.SYSDATETIMEOFFSET", "SqlServer.CURRENT_USER", "SqlServer.CURRENT_TIMESTAMP", "SqlServer.HOST_NAME", "SqlServer.USER_NAME"}
    End Sub

    Friend Sub New(ByVal wrappedCommandDefinition As DbCommandDefinition, ByVal commandTree As DbCommandTree)
        MyBase.New(wrappedCommandDefinition, commandTree, Function(cmd, def) New EFCachingCommand(cmd, def))
        Me.GetAffectedEntitySets(commandTree)
    End Sub

    ''' <summary>
    ''' Gets the list of non-cacheable functions (by default includes canonical and SqlServer functions).
    ''' </summary>
    '''The non-cacheable functions.
    Private Shared privateNonCacheableFunctions As ICollection(Of String)
    Public Shared Property NonCacheableFunctions() As ICollection(Of String)
        Get
            Return privateNonCacheableFunctions
        End Get
        Private Set(ByVal value As ICollection(Of String))
            privateNonCacheableFunctions = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating whether this instance is a modification command (INSERT, UPDATE or DELETE).
    ''' </summary>
    ''' <value>
    ''' Returns <c>true</c> if this instance is modification command (INSERT, UPDATE, DELETE); otherwise, <c>false</c>.
    ''' </value>
    Public Property IsModification() As Boolean

    ''' <summary>
    ''' Gets the list of entity sets affected by this command.
    ''' </summary>
    ''' <value>The affected entity sets.</value>
    Public ReadOnly Property AffectedEntitySets() As IList(Of EntitySetBase)
        Get
            Return Me._affectedEntitySets
        End Get
    End Property

    ''' <summary>
    ''' Creates and returns a <see cref="T:System.Data.Common.DbCommandDefinition"/> object associated with the current connection.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbCommandDefinition"/>.
    ''' </returns>
    Public Overrides Function CreateCommand() As DbCommand
        Return New EFCachingCommand(WrappedCommandDefinition.CreateCommand(), Me)
    End Function

    ''' <summary>
    ''' Determines whether this command definition is cacheable.
    ''' </summary>
    ''' <returns>
    ''' A value of <c>true</c> if this command definition is cacheable; otherwise, <c>false</c>.
    ''' </returns>
    Public Function IsCacheable() As Boolean
        If Not (TypeOf Me.CommandTree Is DbQueryCommandTree) Then
            ' we can only cache queries
            Return False
        End If

        If Me.functionsUsed.Any(Function(f) IsNonDeterministicFunction(f)) Then
            Return False
        End If

        Return True
    End Function

    ''' <summary>
    ''' Determines whether the specified function is non-deterministic.
    ''' </summary>
    ''' <param name="function">The function object.</param>
    ''' <returns>
    ''' A value of <c>true</c> if the function is non-deterministic; otherwise, <c>false</c>.
    ''' </returns>
    Private Shared Function IsNonDeterministicFunction(ByVal [function] As EdmFunction) As Boolean
        Return NonCacheableFunctions.Contains([function].NamespaceName & "." & [function].Name)
    End Function

    Private Sub GetAffectedEntitySets(ByVal commandTree As DbCommandTree)
        Dim visitor As New FindAffectedEntitySetsVisitor(Me._affectedEntitySets, Me.functionsUsed)
        Dim queryTree As DbQueryCommandTree = TryCast(commandTree, DbQueryCommandTree)
        If queryTree IsNot Nothing Then
            queryTree.Query.Accept(visitor)
            Return
        End If

        Dim updateTree As DbUpdateCommandTree = TryCast(commandTree, DbUpdateCommandTree)
        If updateTree IsNot Nothing Then
            Me.IsModification = True
            updateTree.Target.Expression.Accept(visitor)
            updateTree.Predicate.Accept(visitor)
            If updateTree.Returning IsNot Nothing Then
                updateTree.Returning.Accept(visitor)
            End If

            Return
        End If

        Dim insertTree As DbInsertCommandTree = TryCast(commandTree, DbInsertCommandTree)
        If insertTree IsNot Nothing Then
            Me.IsModification = True
            insertTree.Target.Expression.Accept(visitor)
            If insertTree.Returning IsNot Nothing Then
                insertTree.Returning.Accept(visitor)
            End If

            Return
        End If

        Dim deleteTree As DbDeleteCommandTree = TryCast(commandTree, DbDeleteCommandTree)
        If deleteTree IsNot Nothing Then
            Me.IsModification = True
            deleteTree.Target.Expression.Accept(visitor)
            If deleteTree.Predicate IsNot Nothing Then
                deleteTree.Predicate.Accept(visitor)
            End If

            Return
        End If

        Throw New NotSupportedException("Command tree type " & commandTree.GetType().ToString() & " is not supported.")
    End Sub

    ''' <summary>
    ''' Scans the command tree for occurences of entity sets and functions.
    ''' </summary>
    Private Class FindAffectedEntitySetsVisitor
        Inherits DbCommandTreeScanner
        Private affectedEntitySets As ICollection(Of EntitySetBase)
        Private functionsUsed As ICollection(Of EdmFunction)

        ''' <summary>
        ''' Initializes a new instance of the FindAffectedEntitySetsVisitor class.
        ''' </summary>
        ''' <param name="affectedEntitySets">The affected entity sets.</param>
        ''' <param name="functionsUsed">The functions used.</param>
        Public Sub New(ByVal affectedEntitySets As ICollection(Of EntitySetBase), ByVal functionsUsed As ICollection(Of EdmFunction))
            Me.affectedEntitySets = affectedEntitySets
            Me.functionsUsed = functionsUsed
        End Sub

        ''' <summary>
        ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbScanExpression"/>.
        ''' </summary>
        ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbScanExpression"/> that is visited.</param>
        Public Overrides Sub Visit(ByVal expression As DbScanExpression)
            If expression Is Nothing Then
                Throw New ArgumentNullException("expression")
            End If

            MyBase.Visit(expression)
            Me.affectedEntitySets.Add(expression.Target)
        End Sub

        ''' <summary>
        ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbFunctionExpression"/>.
        ''' </summary>
        ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbFunctionExpression"/> that is visited.</param>
        Public Overrides Sub Visit(ByVal expression As DbFunctionExpression)
            If expression Is Nothing Then
                Throw New ArgumentNullException("expression")
            End If

            MyBase.Visit(expression)
            Me.functionsUsed.Add(expression.Function)
        End Sub
    End Class
End Class