﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports Microsoft.VisualBasic
Imports System
Imports System.Data.Common

''' <summary>
''' Base class for caching data readers.
''' </summary>
Friend MustInherit Class EFCachingDataReaderBase
    Inherits DbDataReader
    Private values() As Object

    ''' <summary>
    ''' Initializes a new instance of the EFCachingDataReaderBase class.
    ''' </summary>
    Protected Sub New()
    End Sub

    ''' <summary>
    ''' Gets the number of columns in the current row.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The number of columns in the current row.
    ''' </returns>
    Public Overrides ReadOnly Property FieldCount() As Integer
        Get
            Return Me.values.Length
        End Get
    End Property

    ''' <summary>
    ''' Gets the value of a column with the specified name.
    ''' </summary>
    ''' <value>Value of the column.</value>
    ''' <param name="name">Column name.</param>
    Default Public Overrides ReadOnly Property Item(ByVal name As String) As Object
        Get
            Return Me.GetValue(Me.GetOrdinal(name))
        End Get
    End Property

    ''' <summary>
    ''' Gets the value with the specified ordinal.
    ''' </summary>
    ''' <value>The value at the specified ordinal.</value>
    ''' <param name="ordinal">Column ordinal.</param>
    Default Public Overrides ReadOnly Property Item(ByVal ordinal As Integer) As Object
        Get
            Return Me.GetValue(ordinal)
        End Get
    End Property

    ''' <summary>
    ''' Gets the value of the specified column as a Boolean.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetBoolean(ByVal ordinal As Integer) As Boolean
        Return CBool(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a byte.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetByte(ByVal ordinal As Integer) As Byte
        Return CByte(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Reads a stream of bytes from the specified column, starting at location indicated by <paramref name="dataOffset"/>, into the buffer, starting at the location indicated by <paramref name="bufferOffset"/>.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <param name="dataOffset">The index within the row from which to begin the read operation.</param>
    ''' <param name="buffer">The buffer into which to copy the data.</param>
    ''' <param name="bufferOffset">The index with the buffer to which the data will be copied.</param>
    ''' <param name="length">The maximum number of characters to read.</param>
    ''' <returns>The actual number of bytes read.</returns>
    Public Overrides Function GetBytes(ByVal ordinal As Integer, ByVal dataOffset As Long, ByVal buffer() As Byte, ByVal bufferOffset As Integer, ByVal length As Integer) As Long
        Return CLng(Fix(Me.GetValue(ordinal)))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a single character.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetChar(ByVal ordinal As Integer) As Char
        Return CChar(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Reads a stream of characters from the specified column, starting at location indicated by <paramref name="dataIndex"/>, into the buffer, starting at the location indicated by <paramref name="bufferIndex"/>.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <param name="dataOffset">The index within the row from which to begin the read operation.</param>
    ''' <param name="buffer">The buffer into which to copy the data.</param>
    ''' <param name="bufferOffset">The index with the buffer to which the data will be copied.</param>
    ''' <param name="length">The maximum number of characters to read.</param>
    ''' <returns>The actual number of characters read.</returns>
    Public Overrides Function GetChars(ByVal ordinal As Integer, ByVal dataOffset As Long, ByVal buffer() As Char, ByVal bufferOffset As Integer, ByVal length As Integer) As Long
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a <see cref="T:System.DateTime"/> object.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetDateTime(ByVal ordinal As Integer) As Date
        Return CDate(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a <see cref="T:System.Decimal"/> object.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetDecimal(ByVal ordinal As Integer) As Decimal
        Return CDec(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a double-precision floating point number.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetDouble(ByVal ordinal As Integer) As Double
        Return CDbl(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Returns an <see cref="T:System.Collections.IEnumerator"/> that can be used to iterate through the rows in the data reader.
    ''' </summary>
    ''' <returns>
    ''' An <see cref="T:System.Collections.IEnumerator"/> that can be used to iterate through the rows in the data reader.
    ''' </returns>
    Public Overrides Function GetEnumerator() As System.Collections.IEnumerator
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a single-precision floating point number.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetFloat(ByVal ordinal As Integer) As Single
        Return CSng(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a globally-unique identifier (GUID).
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetGuid(ByVal ordinal As Integer) As Guid
        Return CType(Me.GetValue(ordinal), Guid)
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a 16-bit signed integer.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetInt16(ByVal ordinal As Integer) As Short
        Return CShort(Fix(Me.GetValue(ordinal)))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a 32-bit signed integer.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetInt32(ByVal ordinal As Integer) As Integer
        Return CInt(Fix(Me.GetValue(ordinal)))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as a 64-bit signed integer.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetInt64(ByVal ordinal As Integer) As Long
        Return CLng(Fix(Me.GetValue(ordinal)))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as an instance of <see cref="T:System.String"/>.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetString(ByVal ordinal As Integer) As String
        Return CStr(Me.GetValue(ordinal))
    End Function

    ''' <summary>
    ''' Gets the value of the specified column as an instance of <see cref="T:System.Object"/>.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The value of the specified column.</returns>
    Public Overrides Function GetValue(ByVal ordinal As Integer) As Object
        Return Me.values(ordinal)
    End Function

    ''' <summary>
    ''' Gets all attribute columns in the collection for the current row.
    ''' </summary>
    ''' <param name="result">An array of <see cref="T:System.Object"/> into which to copy the attribute columns.</param>
    ''' <returns>
    ''' The number of instances of <see cref="T:System.Object"/> in the array.
    ''' </returns>
    Public Overrides Function GetValues(ByVal result() As Object) As Integer
        Array.Copy(Me.values, result, Me.values.Length)
        Return Me.values.Length
    End Function

    ''' <summary>
    ''' Gets a value that indicates whether the column contains nonexistent or missing values.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>
    ''' true if the specified column is equivalent to <see cref="T:System.DBNull"/>; otherwise false.
    ''' </returns>
    Public Overrides Function IsDBNull(ByVal ordinal As Integer) As Boolean
        Return TypeOf Me.values(ordinal) Is DBNull
    End Function

    ''' <summary>
    ''' Sets the collection of values that the reader should return.
    ''' </summary>
    ''' <param name="rowValues">The values.</param>
    Protected Sub SetValues(ByVal rowValues() As Object)
        Me.values = rowValues
    End Sub
End Class