﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data
Imports System.Data.Common
Imports EFCachingProvider.Caching

''' <summary>
''' Implementation of the <see cref="DbDataReader"/> which reads the results of another data reader
''' and stores a copy in the cache.
''' </summary>
Friend Class EFCachingDataReaderCacheWriter
    Inherits EFCachingDataReaderBase
    Private queryResults As New DbQueryResults()
    Private wrappedReader As DbDataReader
    Private maxRows As Integer
    Private addToCache As Action(Of DbQueryResults)

    ''' <summary>
    ''' Initializes a new instance of the EFCachingDataReaderCacheWriter class.
    ''' </summary>
    ''' <param name="wrappedReader">The wrapped reader.</param>
    ''' <param name="maxRows">The maximum number of rows which can be cached.</param>
    ''' <param name="addToCache">The delegate used to add the result to the cache when the reader finishes.</param>
    Public Sub New(ByVal wrappedReader As DbDataReader, ByVal maxRows As Integer, ByVal addToCache As Action(Of DbQueryResults))
        Me.wrappedReader = wrappedReader
        Me.addToCache = addToCache
        Me.maxRows = maxRows
        For i = 0 To wrappedReader.FieldCount - 1
            Me.queryResults.ColumnNames.Add(wrappedReader.GetName(i))
        Next i
    End Sub

    ''' <summary>
    ''' Gets a value that indicates whether this <see cref="T:System.Data.Common.DbDataReader"/> contains one or more rows.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the <see cref="T:System.Data.Common.DbDataReader"/> contains one or more rows; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property HasRows() As Boolean
        Get
            Return Me.wrappedReader.HasRows
        End Get
    End Property

    ''' <summary>
    ''' Gets the number of rows changed, inserted, or deleted by execution of the SQL statement.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The number of rows changed, inserted, or deleted. -1 for SELECT statements; 0 if no rows were affected or the statement failed.
    ''' </returns>
    Public Overrides ReadOnly Property RecordsAffected() As Integer
        Get
            Return Me.wrappedReader.RecordsAffected
        End Get
    End Property

    ''' <summary>
    ''' Gets a value indicating whether the <see cref="T:System.Data.Common.DbDataReader"/> is closed.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the <see cref="T:System.Data.Common.DbDataReader"/> is closed; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property IsClosed() As Boolean
        Get
            Return Me.wrappedReader.IsClosed
        End Get
    End Property

    ''' <summary>
    ''' Gets a value indicating the depth of nesting for the current row.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The depth of nesting for the current row.
    ''' </returns>
    Public Overrides ReadOnly Property Depth() As Integer
        Get
            Return Me.wrappedReader.Depth
        End Get
    End Property

    ''' <summary>
    ''' Gets name of the data type of the specified column.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>
    ''' A string representing the name of the data type.
    ''' </returns>
    Public Overrides Function GetDataTypeName(ByVal ordinal As Integer) As String
        Return Me.wrappedReader.GetDataTypeName(ordinal)
    End Function

    ''' <summary>
    ''' Gets the data type of the specified column.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The data type of the specified column.</returns>
    Public Overrides Function GetFieldType(ByVal ordinal As Integer) As Type
        Return Me.wrappedReader.GetFieldType(ordinal)
    End Function

    ''' <summary>
    ''' Gets the name of the column, given the zero-based column ordinal.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The name of the specified column.</returns>
    Public Overrides Function GetName(ByVal ordinal As Integer) As String
        Return Me.wrappedReader.GetName(ordinal)
    End Function

    ''' <summary>
    ''' Gets the column ordinal given the name of the column.
    ''' </summary>
    ''' <param name="name">The name of the column.</param>
    ''' <returns>The zero-based column ordinal.</returns>
    ''' <exception cref="T:System.IndexOutOfRangeException">
    ''' The name specified is not a valid column name.
    ''' </exception>
    Public Overrides Function GetOrdinal(ByVal name As String) As Integer
        Return Me.wrappedReader.GetOrdinal(name)
    End Function

    ''' <summary>
    ''' Returns a <see cref="T:System.Data.DataTable"/> that describes the column metadata of the <see cref="T:System.Data.Common.DbDataReader"/>.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.DataTable"/> that describes the column metadata.
    ''' </returns>
    Public Overrides Function GetSchemaTable() As DataTable
        Return Me.wrappedReader.GetSchemaTable()
    End Function

    ''' <summary>
    ''' Advances the reader to the next result when reading the results of a batch of statements.
    ''' </summary>
    ''' <returns>
    ''' true if there are more result sets; otherwise false.
    ''' </returns>
    Public Overrides Function NextResult() As Boolean
        If Me.wrappedReader.NextResult() Then
            Me.queryResults = Nothing
            Return True
        End If

        Return False
    End Function

    ''' <summary>
    ''' Closes the <see cref="T:System.Data.Common.DbDataReader"/> object.
    ''' </summary>
    Public Overrides Sub Close()
        Me.wrappedReader.Close()

        If Me.queryResults IsNot Nothing Then
            Me.addToCache(Me.queryResults)
        End If
    End Sub

    ''' <summary>
    ''' Advances the reader to the next record in a result set.
    ''' </summary>
    ''' <returns>
    ''' true if there are more rows; otherwise false.
    ''' </returns>
    Public Overrides Function Read() As Boolean
        If Me.wrappedReader.Read() Then
            Dim values = New Object(Me.wrappedReader.FieldCount - 1) {}

            Me.wrappedReader.GetValues(values)
            SetValues(values)
            If Me.queryResults IsNot Nothing Then
                Me.queryResults.Rows.Add(values)
                If Me.queryResults.Rows.Count > Me.maxRows Then
                    Me.queryResults = Nothing
                End If
            End If

            Return True
        End If

        Return False
    End Function
End Class