﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Collections.Generic
Imports System.Data.Common
Imports System.Data.Metadata.Edm
Imports System.Linq
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbTransaction"/> for EFCachingProvider.
''' </summary>
''' <remarks>
''' Keeps track of all tables affected by the transaction and invalidates queries dependent on those tables
''' from the cache.
''' </remarks>
Public NotInheritable Class EFCachingTransaction
    Inherits DbTransactionWrapper
    Private affectedEntitySets As New HashSet(Of EntitySetBase)()

    ''' <summary>
    ''' Initializes a new instance of the EFCachingTransaction class.
    ''' </summary>
    ''' <param name="transaction">The transaction.</param>
    ''' <param name="connection">The connection.</param>
    Public Sub New(ByVal transaction As DbTransaction, ByVal connection As EFCachingConnection)
        MyBase.New(transaction, connection)
    End Sub

    ''' <summary>
    ''' Gets or sets a value indicating whether this transaction includes modification commands (INSERT, UPDATE, DELETE).
    ''' </summary>
    ''' <value>
    ''' A value of <c>true</c> if this instance includes modification commands; otherwise, <c>false</c>.
    ''' </value>
    Public Property HasModifications() As Boolean

    ''' <summary>
    ''' Gets the <see cref="T:System.Data.Common.DbConnection"/> object associated with the transaction.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The <see cref="T:System.Data.Common.DbConnection"/> object associated with the transaction.
    ''' </returns>
    Private Shadows ReadOnly Property Connection() As EFCachingConnection
        Get
            Return CType(MyBase.Connection, EFCachingConnection)
        End Get
    End Property

    ''' <summary>
    ''' Commits the database transaction.
    ''' </summary>
    Public Overrides Sub Commit()
        MyBase.Commit()
        Dim cache As ICache = Me.Connection.Cache
        If cache IsNot Nothing AndAlso Me.HasModifications Then
            cache.InvalidateSets(Me.affectedEntitySets.Select(Function(c) c.Name))
        End If
    End Sub

    ''' <summary>
    ''' Marks the specified entity set as affected by the transaction.
    ''' </summary>
    ''' <param name="entitySet">The entity set.</param>
    Public Sub AddAffectedEntitySet(ByVal entitySet As EntitySetBase)
        Me.affectedEntitySets.Add(entitySet)
    End Sub
End Class