﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data
Imports System.Data.Common

''' <summary>
''' DbCommand wrapper base class.
''' </summary>
Public Class DbCommandWrapper
    Inherits DbCommand

    Private _wrappedCommand As DbCommand

    Private _connection As DbConnection

    Private _definition As DbCommandDefinitionWrapper

    ''' <summary>
    ''' Initializes a new instance of the <see cref="DbCommandWrapper"/> class.
    ''' </summary>
    ''' <param name="wrappedCommand">The wrapped command.</param>
    ''' <param name="definition">The definition.</param>
    Public Sub New(ByVal wrappedCommand As DbCommand, ByVal definition As DbCommandDefinitionWrapper)
        Me._wrappedCommand = wrappedCommand
        Me._definition = definition
    End Sub

    ''' <summary>
    ''' Gets the command definition.
    ''' </summary>
    ''' <value>The command definition.</value>
    Public ReadOnly Property Definition() As DbCommandDefinitionWrapper
        Get
            Return Me._definition
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the text command to run against the data source.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The text command to execute. The default value is an empty string ("").
    ''' </returns>
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2100:Review SQL queries for security vulnerabilities", Justification:="This is a pass-through")>
    Public Overrides Property CommandText() As String
        Get
            Return Me._wrappedCommand.CommandText
        End Get
        Set(ByVal value As String)
            Me._wrappedCommand.CommandText = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the wait time before terminating the attempt to execute a command and generating an error.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The time in seconds to wait for the command to execute.
    ''' </returns>
    Public Overrides Property CommandTimeout() As Integer
        Get
            Return Me._wrappedCommand.CommandTimeout
        End Get
        Set(ByVal value As Integer)
            Me._wrappedCommand.CommandTimeout = value
        End Set
    End Property

    ''' <summary>
    ''' Indicates or specifies how the <see cref="P:System.Data.Common.DbCommand.CommandText"/> property is interpreted.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' One of the <see cref="T:System.Data.CommandType"/> values. The default is Text.
    ''' </returns>
    Public Overrides Property CommandType() As CommandType
        Get
            Return Me._wrappedCommand.CommandType
        End Get
        Set(ByVal value As CommandType)
            Me._wrappedCommand.CommandType = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets a value indicating whether the command object should be visible in a customized interface control.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true, if the command object should be visible in a control; otherwise false. The default is true.
    ''' </returns>
    Public Overrides Property DesignTimeVisible() As Boolean
        Get
            Return False
        End Get
        Set(ByVal value As Boolean)
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets how command results are applied to the <see cref="T:System.Data.DataRow"/> when used by the Update method of a <see cref="T:System.Data.Common.DbDataAdapter"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' One of the <see cref="T:System.Data.UpdateRowSource"/> values. The default is Both unless the command is automatically generated. Then the default is None.
    ''' </returns>
    Public Overrides Property UpdatedRowSource() As UpdateRowSource
        Get
            Return Me._wrappedCommand.UpdatedRowSource
        End Get
        Set(ByVal value As UpdateRowSource)
            Me._wrappedCommand.UpdatedRowSource = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the wrapped command.
    ''' </summary>
    ''' <value>The wrapped command.</value>
    Protected ReadOnly Property WrappedCommand() As DbCommand
        Get
            Return Me._wrappedCommand
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the <see cref="T:System.Data.Common.DbConnection"/> used by this <see cref="T:System.Data.Common.DbCommand"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The connection to the data source.
    ''' </returns>
    Protected Overrides Property DbConnection() As DbConnection
        Get
            Return Me._connection
        End Get

        Set(ByVal value As DbConnection)
            Dim conn As DbConnectionWrapper = CType(value, DbConnectionWrapper)
            If conn IsNot Nothing Then
                Me._connection = conn
                Me._wrappedCommand.Connection = conn.WrappedConnection
            Else
                Me._connection = Nothing
                Me._wrappedCommand.Connection = Nothing
            End If
        End Set
    End Property

    ''' <summary>
    ''' Gets the collection of <see cref="T:System.Data.Common.DbParameter"/> objects.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The parameters of the SQL statement or stored procedure.
    ''' </returns>
    Protected Overrides ReadOnly Property DbParameterCollection() As DbParameterCollection
        Get
            Return Me._wrappedCommand.Parameters
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the <see cref="P:System.Data.Common.DbCommand.DbTransaction"/> within which this <see cref="T:System.Data.Common.DbCommand"/> object executes.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The transaction within which a Command object of a .NET Framework data provider executes. The default value is a null reference (Nothing in Visual Basic).
    ''' </returns>
    Protected Overrides Property DbTransaction() As DbTransaction
        Get
            Return Me._wrappedCommand.Transaction
        End Get
        Set(ByVal value As DbTransaction)
            Me._wrappedCommand.Transaction = value
        End Set
    End Property

    ''' <summary>
    ''' Attempts to cancels the execution of a <see cref="T:System.Data.Common.DbCommand"/>.
    ''' </summary>
    Public Overrides Sub Cancel()
        Throw New NotImplementedException()
    End Sub

    ''' <summary>
    ''' Executes a SQL statement against a connection object.
    ''' </summary>
    ''' <returns>The number of rows affected.</returns>
    Public Overrides Function ExecuteNonQuery() As Integer
        Return Me._wrappedCommand.ExecuteNonQuery()
    End Function

    ''' <summary>
    ''' Executes the query and returns the first column of the first row in the result set returned by the query. All other columns and rows are ignored.
    ''' </summary>
    ''' <returns>
    ''' The first column of the first row in the result set.
    ''' </returns>
    Public Overrides Function ExecuteScalar() As Object
        Return Me._wrappedCommand.ExecuteScalar()
    End Function

    ''' <summary>
    ''' Creates a prepared (or compiled) version of the command on the data source.
    ''' </summary>
    Public Overrides Sub Prepare()
        Me._wrappedCommand.Prepare()
    End Sub

    ''' <summary>
    ''' Creates a new instance of a <see cref="T:System.Data.Common.DbParameter"/> object.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbParameter"/> object.
    ''' </returns>
    Protected Overrides Function CreateDbParameter() As DbParameter
        Return Me._wrappedCommand.CreateParameter()
    End Function

    ''' <summary>
    ''' Executes the command text against the connection.
    ''' </summary>
    ''' <param name="behavior">An instance of <see cref="T:System.Data.CommandBehavior"/>.</param>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbDataReader"/>.
    ''' </returns>
    Protected Overrides Function ExecuteDbDataReader(ByVal behavior As CommandBehavior) As DbDataReader
        Return Me._wrappedCommand.ExecuteReader(behavior)
    End Function
End Class