﻿

' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data
Imports System.Data.Common

''' <summary>
''' Wrapper for <see cref="DbConnection"/> objects.
''' </summary>
Public MustInherit Class DbConnectionWrapper
    Inherits DbConnection
    Private Shared connectionStringKey As String = "wrappedProvider="

    Private _wrappedConnection As DbConnection

    Private _wrappedProviderInvariantName As String

    ''' <summary>
    ''' Initializes a new instance of the DbConnectionWrapper class.
    ''' </summary>
    Protected Sub New()
    End Sub

    ''' <summary>
    ''' Gets the time to wait while establishing a connection before terminating the attempt and generating an error.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The time (in seconds) to wait for a connection to open. The default value is determined by the specific type of connection that you are using.
    ''' </returns>
    Public Overrides ReadOnly Property ConnectionTimeout() As Integer
        Get
            Return Me._wrappedConnection.ConnectionTimeout
        End Get
    End Property

    ''' <summary>
    ''' Gets the name of the current database after a connection is opened, or the database name specified in the connection string before the connection is opened.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The name of the current database or the name of the database to be used after a connection is opened. The default value is an empty string.
    ''' </returns>
    Public Overrides ReadOnly Property Database() As String
        Get
            Return Me._wrappedConnection.Database
        End Get
    End Property

    ''' <summary>
    ''' Gets the name of the database server to which to connect.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The name of the database server to which to connect. The default value is an empty string.
    ''' </returns>
    Public Overrides ReadOnly Property DataSource() As String
        Get
            Return Me._wrappedConnection.DataSource
        End Get
    End Property

    ''' <summary>
    ''' Gets the invariant name of the wrapped provider.
    ''' </summary>
    ''' <value>The invariant name of the wrapped provider.</value>
    Public ReadOnly Property WrappedProviderInvariantName() As String
        Get
            Return Me._wrappedProviderInvariantName
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the wrapped connection.
    ''' </summary>
    ''' <value>The wrapped connection.</value>
    Public Property WrappedConnection() As DbConnection
        Get
            Return Me._wrappedConnection
        End Get
        Set(ByVal value As DbConnection)
            Me._wrappedConnection = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the string used to open the connection.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The connection string used to establish the initial connection. The exact contents of the connection string depend on the specific data source for this connection. The default value is an empty string.
    ''' </returns>
    Public Overrides Property ConnectionString() As String
        Get
            Return connectionStringKey & Me._wrappedProviderInvariantName & ";" & Me._wrappedConnection.ConnectionString
        End Get

        Set(ByVal value As String)
            Me.CreateConnectionFromConnectionString(value)
        End Set
    End Property

    ''' <summary>
    ''' Gets a string that represents the version of the server to which the object is connected.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The version of the database. The format of the string returned depends on the specific type of connection you are using.
    ''' </returns>
    Public Overrides ReadOnly Property ServerVersion() As String
        Get
            Return Me._wrappedConnection.ServerVersion
        End Get
    End Property

    ''' <summary>
    ''' Gets or sets the <see cref="T:System.ComponentModel.ISite"/> of the <see cref="T:System.ComponentModel.Component"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The <see cref="T:System.ComponentModel.ISite"/> associated with the <see cref="T:System.ComponentModel.Component"/>, or null if the <see cref="T:System.ComponentModel.Component"/> is not encapsulated in an <see cref="T:System.ComponentModel.IContainer"/>, the <see cref="T:System.ComponentModel.Component"/> does not have an <see cref="T:System.ComponentModel.ISite"/> associated with it, or the <see cref="T:System.ComponentModel.Component"/> is removed from its <see cref="T:System.ComponentModel.IContainer"/>.
    ''' </returns>
    Public Overrides Property Site() As System.ComponentModel.ISite
        Get
            Return Me._wrappedConnection.Site
        End Get
        Set(ByVal value As System.ComponentModel.ISite)
            Me._wrappedConnection.Site = value
        End Set
    End Property

    ''' <summary>
    ''' Gets a string that describes the state of the connection.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The state of the connection. The format of the string returned depends on the specific type of connection you are using.
    ''' </returns>
    Public Overrides ReadOnly Property State() As ConnectionState
        Get
            Return Me._wrappedConnection.State
        End Get
    End Property

    ''' <summary>
    ''' Gets the name of the default wrapped provider.
    ''' </summary>
    ''' <returns>Name of the default wrapped provider.</returns>
    Protected MustOverride ReadOnly Property DefaultWrappedProviderName() As String

    ''' <summary>
    ''' Enlists in the specified transaction.
    ''' </summary>
    ''' <param name="transaction">A reference to an existing <see cref="T:System.Transactions.Transaction"/> in which to enlist.</param>
    Public Overrides Sub EnlistTransaction(ByVal transaction As System.Transactions.Transaction)
        Me._wrappedConnection.EnlistTransaction(transaction)
    End Sub

    ''' <summary>
    ''' Returns schema information for the data source of this <see cref="T:System.Data.Common.DbConnection"/> using the specified string for the schema name.
    ''' </summary>
    ''' <param name="collectionName">Specifies the name of the schema to return.</param>
    ''' <returns>
    ''' A <see cref="T:System.Data.DataTable"/> that contains schema information.
    ''' </returns>
    Public Overloads Overrides Function GetSchema(ByVal collectionName As String) As DataTable
        Return Me._wrappedConnection.GetSchema(collectionName)
    End Function

    ''' <summary>
    ''' Returns schema information for the data source of this <see cref="T:System.Data.Common.DbConnection"/>.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.DataTable"/> that contains schema information.
    ''' </returns>
    Public Overloads Overrides Function GetSchema() As DataTable
        Return Me._wrappedConnection.GetSchema()
    End Function

    ''' <summary>
    ''' Returns schema information for the data source of this <see cref="T:System.Data.Common.DbConnection"/> using the specified string for the schema name and the specified string array for the restriction values.
    ''' </summary>
    ''' <param name="collectionName">Specifies the name of the schema to return.</param>
    ''' <param name="restrictionValues">Specifies a set of restriction values for the requested schema.</param>
    ''' <returns>
    ''' A <see cref="T:System.Data.DataTable"/> that contains schema information.
    ''' </returns>
    Public Overloads Overrides Function GetSchema(ByVal collectionName As String, ByVal restrictionValues() As String) As DataTable
        Return Me._wrappedConnection.GetSchema(collectionName, restrictionValues)
    End Function

    ''' <summary>
    ''' Opens a database connection with the settings specified by the <see cref="P:System.Data.Common.DbConnection.ConnectionString"/>.
    ''' </summary>
    Public Overrides Sub Open()
        Me._wrappedConnection.Open()
    End Sub

    ''' <summary>
    ''' Changes the current database for an open connection.
    ''' </summary>
    ''' <param name="databaseName">Specifies the name of the database for the connection to use.</param>
    Public Overrides Sub ChangeDatabase(ByVal databaseName As String)
        Me._wrappedConnection.ChangeDatabase(databaseName)
    End Sub

    ''' <summary>
    ''' Closes the connection to the database. This is the preferred method of closing any open connection.
    ''' </summary>
    ''' <exception cref="T:System.Data.Common.DbException">
    ''' The connection-level error that occurred while opening the connection.
    ''' </exception>
    Public Overrides Sub Close()
        Me._wrappedConnection.Close()
    End Sub

    ''' <summary>
    ''' Wraps the connection.
    ''' </summary>
    ''' <param name="connection">The connection.</param>
    ''' <param name="wrapperProviderInvariantNames">The wrapper provider invariant names.</param>
    ''' <returns>Wrapped connection.</returns>
    Friend Shared Function WrapConnection(ByVal connection As DbConnection, ByVal ParamArray wrapperProviderInvariantNames() As String) As DbConnection
        For Each invariantName In wrapperProviderInvariantNames
            Dim factory As DbProviderFactory = DbProviderFactories.GetFactory(invariantName)
            Dim connectionWrapper = factory.CreateConnection()
            Dim wrapper As DbConnectionWrapper = CType(connectionWrapper, DbConnectionWrapper)
            wrapper.WrappedConnection = connection
            connection = connectionWrapper
        Next invariantName

        Return connection
    End Function

    ''' <summary>
    ''' Starts a database transaction.
    ''' </summary>
    ''' <param name="isolationLevel">Specifies the isolation level for the transaction.</param>
    ''' <returns>
    ''' An object representing the new transaction.
    ''' </returns>
    Protected Overrides Function BeginDbTransaction(ByVal isolationLevel As System.Data.IsolationLevel) As DbTransaction
        Return Me._wrappedConnection.BeginTransaction(isolationLevel)
    End Function

    ''' <summary>
    ''' Releases the unmanaged resources used by the <see cref="T:System.ComponentModel.Component"/> and optionally releases the managed resources.
    ''' </summary>
    ''' <param name="disposing">true to release both managed and unmanaged resources; false to release only unmanaged resources.</param>
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            Me._wrappedConnection.Dispose()
        End If

        MyBase.Dispose(disposing)
    End Sub

    ''' <summary>
    ''' Creates and returns a <see cref="T:System.Data.Common.DbCommand"/> object associated with the current connection.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbCommand"/> object.
    ''' </returns>
    Protected Overrides Function CreateDbCommand() As DbCommand
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Creates the connection from connection string.
    ''' </summary>
    ''' <param name="connectionString">The connection string.</param>
    Private Sub CreateConnectionFromConnectionString(ByVal connectionString As String)
        Dim providerInvariantName As String
        Dim newConnectionString As String

        Dim pos As Integer = connectionString.IndexOf(connectionStringKey, StringComparison.OrdinalIgnoreCase)
        If pos = 0 Then
            Dim pos2 As Integer = connectionString.IndexOf(";", pos, StringComparison.Ordinal)
            If pos2 < 0 Then
                Throw New ArgumentException("Invalid connection string format")
            End If

            providerInvariantName = connectionString.Substring(connectionStringKey.Length, pos2 - connectionStringKey.Length)
            newConnectionString = connectionString.Substring(pos2 + 1)
        Else
            providerInvariantName = Me.DefaultWrappedProviderName
            newConnectionString = connectionString
        End If

        Dim factory As DbProviderFactory = DbProviderFactories.GetFactory(providerInvariantName)
        Me._wrappedProviderInvariantName = providerInvariantName
        Me._wrappedConnection = factory.CreateConnection()
        Me._wrappedConnection.ConnectionString = newConnectionString
    End Sub
End Class