﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Configuration
Imports System.Data
Imports System.Data.Common

''' <summary>
''' Common implementation of the <see cref="DbProviderFactory"/> methods.
''' </summary>
<CLSCompliant(False)>
Public MustInherit Class DbProviderFactoryBase
    Inherits DbProviderFactory
    Implements IServiceProvider
    Private services As DbProviderServices

    ''' <summary>
    ''' Initializes a new instance of the DbProviderFactoryBase class.
    ''' </summary>
    ''' <param name="providerServices">The provider services.</param>
    Protected Sub New(ByVal providerServices As DbProviderServices)
        Me.services = providerServices
    End Sub

    ''' <summary>
    ''' Specifies whether the specific <see cref="T:System.Data.Common.DbProviderFactory"/> supports the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the instance of the <see cref="T:System.Data.Common.DbProviderFactory"/> supports the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property CanCreateDataSourceEnumerator() As Boolean
        Get
            Throw New NotSupportedException()
        End Get
    End Property

    ''' <summary>
    ''' Registers the provider.
    ''' </summary>
    ''' <param name="name">Human-readable name of the provider.</param>
    ''' <param name="invariantName">Invariant name of the provider.</param>
    ''' <param name="factoryType">Factory type for the provider.</param>
    Public Shared Sub RegisterProvider(ByVal name As String, ByVal invariantName As String, ByVal factoryType As Type)
        If String.IsNullOrEmpty(name) Then
            Throw New ArgumentNullException("name")
        End If

        If String.IsNullOrEmpty(invariantName) Then
            Throw New ArgumentNullException("invariantName")
        End If

        If factoryType Is Nothing Then
            Throw New ArgumentNullException("factoryType")
        End If

        Dim data = CType(ConfigurationManager.GetSection("system.data"), DataSet)
        Dim providerFactories = data.Tables("DbProviderFactories")
        providerFactories.Rows.Add(name, name, invariantName, factoryType.AssemblyQualifiedName)
    End Sub

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbCommandBuilder"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbCommandBuilder"/>.
    ''' </returns>
    Public Overrides Function CreateCommandBuilder() As DbCommandBuilder
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbConnectionStringBuilder"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbConnectionStringBuilder"/>.
    ''' </returns>
    Public Overrides Function CreateConnectionStringBuilder() As DbConnectionStringBuilder
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbDataAdapter"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbDataAdapter"/>.
    ''' </returns>
    Public Overrides Function CreateDataAdapter() As DbDataAdapter
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbDataSourceEnumerator"/>.
    ''' </returns>
    Public Overrides Function CreateDataSourceEnumerator() As DbDataSourceEnumerator
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbParameter"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbParameter"/>.
    ''' </returns>
    Public Overrides Function CreateParameter() As DbParameter
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbConnection"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbConnection"/>.
    ''' </returns>
    Public MustOverride Overrides Function CreateConnection() As DbConnection

    ''' <summary>
    ''' Gets the service object of the specified type.
    ''' </summary>
    ''' <param name="serviceType">An object that specifies the type of service object to get.</param>
    ''' <returns>
    ''' A service object of type <paramref name="serviceType"/>.
    ''' -or-
    ''' null if there is no service object of type <paramref name="serviceType"/>.
    ''' </returns>
    Public Function GetService(ByVal serviceType As Type) As Object Implements IServiceProvider.GetService
        If serviceType Is GetType(DbProviderServices) Then
            Return Me.services
        Else
            Return Nothing
        End If
    End Function
End Class