﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Data.Common
Imports System.Data.Common.CommandTrees
Imports System.Data.Metadata.Edm

''' <summary>
''' Common implementation of <see cref="DbProviderServices"/> methods.
''' </summary>
Public MustInherit Class DbProviderServicesBase
    Inherits DbProviderServices
    Private createDbCommandDefinitionFunctions As New Dictionary(Of String, Func(Of DbProviderManifest, DbCommandTree, DbCommandDefinition))()

    ''' <summary>
    ''' Gets the provider invariant iname.
    ''' </summary>
    ''' <returns>Provider invariant name.</returns>
    Protected MustOverride ReadOnly Property ProviderInvariantName() As String

    ''' <summary>
    ''' Gets the default name of the wrapped provider.
    ''' </summary>
    ''' <returns>Default name of the wrapped provider (to be used when 
    ''' provider is not specified in the connction string)</returns>
    Protected MustOverride ReadOnly Property DefaultWrappedProviderName() As String

    ''' <summary>
    ''' Creates the command definition wrapper.
    ''' </summary>
    ''' <param name="wrappedCommandDefinition">The wrapped command definition.</param>
    ''' <param name="commandTree">The command tree.</param>
    ''' <returns><see cref="DbCommandDefinitionWrapper"/> object.</returns>
    Public Overridable Function CreateCommandDefinitionWrapper(ByVal wrappedCommandDefinition As DbCommandDefinition, ByVal commandTree As DbCommandTree) As DbCommandDefinitionWrapper
        Return New DbCommandDefinitionWrapper(wrappedCommandDefinition, commandTree, Function(cmd, def) New DbCommandWrapper(cmd, def))
    End Function

    ''' <summary>
    ''' Creates the provider manifest wrapper.
    ''' </summary>
    ''' <param name="providerInvariantName">Provider invariant name.</param>
    ''' <param name="providerManifest">The provider manifest to be wrapped.</param>
    ''' <returns><see cref="DbProviderManifest"/> wrapper for given provider invariant name wrapping given provider manifest.</returns>
    Public Overridable Function CreateProviderManifest(ByVal providerInvariantName As String, ByVal providerManifest As DbProviderManifest) As DbProviderManifest
        Return New DbProviderManifestWrapper(Me.ProviderInvariantName, providerInvariantName, providerManifest)
    End Function

    ''' <summary>
    ''' Gets the provider manifest token.
    ''' </summary>
    ''' <param name="connection">The database connection.</param>
    ''' <returns>Provider Manfiest Token suitable for inclusion in SSDL file and connection string</returns>
    ''' <remarks>
    ''' The provider manifest token is created by concatenating wrapped provider invariant name and its 
    ''' token separated by semicolon, for example when wrapping SqlClient for SQL Server 2005 the provider 
    ''' manifest token will be "System.Data.SqlClient;2005"
    ''' </remarks>
    Protected Overrides Function GetDbProviderManifestToken(ByVal connection As DbConnection) As String
        Dim wrapper As DbConnectionWrapper = CType(connection, DbConnectionWrapper)
        Dim wrappedConnection As DbConnection = wrapper.WrappedConnection
        Dim services As DbProviderServices = DbProviderServices.GetProviderServices(wrappedConnection)

        Dim token As String = wrapper.WrappedProviderInvariantName & ";" & services.GetProviderManifestToken(wrappedConnection)
        Return token
    End Function

    ''' <summary>
    ''' Gets the provider manifest for given provider manifest token.
    ''' </summary>
    ''' <param name="manifestToken">The provider manifest token.</param>
    ''' <returns><see cref="DbProviderManifest"/> for a given token.</returns>
    Protected Overrides Function GetDbProviderManifest(ByVal manifestToken As String) As DbProviderManifest
        If String.IsNullOrEmpty(manifestToken) Then
            Throw New ArgumentNullException("manifestToken")
        End If

        Dim providerInvariantName As String
        Dim realToken As String

        ' check if the name of the wrapped provider is specified in the token
        Dim p As Integer = manifestToken.IndexOf(";"c)
        If p < 0 Then
            ' wrapped provider is not in the token - use default one
            realToken = manifestToken
            providerInvariantName = Me.DefaultWrappedProviderName
        Else
            ' extract provider name from the token
            providerInvariantName = manifestToken.Substring(0, p)
            realToken = manifestToken.Substring(p + 1)
        End If

        ' retrieve wrapped provider manifest
        Dim services As DbProviderServices = GetProviderServicesByName(providerInvariantName)
        Dim wrappedProviderManifest As DbProviderManifest = services.GetProviderManifest(realToken)
        Dim wrapperManifest As DbProviderManifest = Me.CreateProviderManifest(providerInvariantName, wrappedProviderManifest)

        Return wrapperManifest
    End Function

    ''' <summary>
    ''' Creates the command definition wrapper for a given provider manifest and command tree.
    ''' </summary>
    ''' <param name="providerManifest">The provider manifest.</param>
    ''' <param name="commandTree">The command tree.</param>
    ''' <returns><see cref="DbCommandDefinition"/> object.</returns>
    Protected Overrides Function CreateDbCommandDefinition(ByVal providerManifest As DbProviderManifest, ByVal commandTree As DbCommandTree) As DbCommandDefinition
        Dim wrapperManifest = CType(providerManifest, DbProviderManifestWrapper)
        Dim createDbCommandDefinitionFunction = Me.GetCreateDbCommandDefinitionFunction(wrapperManifest.WrappedProviderManifestInvariantName)

        Dim definition As DbCommandDefinition = createDbCommandDefinitionFunction(wrapperManifest.WrappedProviderManifest, commandTree)
        Return Me.CreateCommandDefinitionWrapper(definition, commandTree)
    End Function

    ''' <summary>
    ''' Creates a database indicated by connection and creates schema objects (tables, primary keys, foreign keys) based on the contents of a <see cref="T:System.Data.Metadata.Edm.StoreItemCollection"/>.
    ''' </summary>
    ''' <param name="connection">Connection to a non-existent database that needs to be created and populated with the store objects indicated with the storeItemCollection parameter.</param>
    ''' <param name="commandTimeout">Execution timeout for any commands needed to create the database.</param>
    ''' <param name="storeItemCollection">The collection of all store items based on which the script should be created.</param>
    Protected Overrides Sub DbCreateDatabase(ByVal connection As DbConnection, ByVal commandTimeout? As Integer, ByVal storeItemCollection As StoreItemCollection)
        Dim wrappedConnection = (CType(connection, DbConnectionWrapper)).WrappedConnection
        Dim services = DbProviderServices.GetProviderServices(wrappedConnection)

        services.CreateDatabase(wrappedConnection, commandTimeout, storeItemCollection)
    End Sub

    ''' <summary>
    ''' Returns a value indicating whether a given database exists on the server and whether schema objects contained in the storeItemCollection have been created.
    ''' </summary>
    ''' <param name="connection">Connection to a database whose existence is verified by this method.</param>
    ''' <param name="commandTimeout">Execution timeout for any commands needed to determine the existence of the database.</param>
    ''' <param name="storeItemCollection">The structure of the database whose existence is determined by this method.</param>
    ''' <returns>
    ''' true if the database indicated by the connection and the <paramref name="storeItemCollection"/> parameter exists.
    ''' </returns>
    Protected Overrides Function DbDatabaseExists(ByVal connection As DbConnection, ByVal commandTimeout? As Integer, ByVal storeItemCollection As StoreItemCollection) As Boolean
        Dim wrappedConnection = (CType(connection, DbConnectionWrapper)).WrappedConnection
        Dim services = DbProviderServices.GetProviderServices(wrappedConnection)

        Return services.DatabaseExists(wrappedConnection, commandTimeout, storeItemCollection)
    End Function

    ''' <summary>
    ''' Deletes all store objects specified in the store item collection from the database and the database itself.
    ''' </summary>
    ''' <param name="connection">Connection to an existing database that needs to be deleted.</param>
    ''' <param name="commandTimeout">Execution timeout for any commands needed to delete the database.</param>
    ''' <param name="storeItemCollection">The structure of the database to be deleted.</param>
    Protected Overrides Sub DbDeleteDatabase(ByVal connection As DbConnection, ByVal commandTimeout? As Integer, ByVal storeItemCollection As StoreItemCollection)
        Dim wrappedConnection = (CType(connection, DbConnectionWrapper)).WrappedConnection
        Dim services = DbProviderServices.GetProviderServices(wrappedConnection)

        services.DeleteDatabase(wrappedConnection, commandTimeout, storeItemCollection)
    End Sub

    ''' <summary>
    ''' Generates a data definition langauge (DDL0 script that creates schema objects (tables, primary keys, foreign keys) based on the contents of the <see cref="T:System.Data.Metadata.Edm.StoreItemCollection"/> parameter and targeted for the version of the database corresponding to the provider manifest token.
    ''' </summary>
    ''' <param name="providerManifestToken">The provider manifest token identifying the target version.</param>
    ''' <param name="storeItemCollection">The structure of the database.</param>
    ''' <returns>
    ''' A DDL script that creates schema objects based on the contents of the <see cref="T:System.Data.Metadata.Edm.StoreItemCollection"/> parameter and targeted for the version of the database corresponding to the provider manifest token.
    ''' </returns>
    Protected Overrides Function DbCreateDatabaseScript(ByVal providerManifestToken As String, ByVal storeItemCollection As StoreItemCollection) As String
        If String.IsNullOrEmpty(providerManifestToken) Then
            Throw New ArgumentNullException("providerManifestToken")
        End If

        Dim providerInvariantName As String
        Dim realToken As String

        ' check if the name of the wrapped provider is specified in the token
        Dim p As Integer = providerManifestToken.IndexOf(";"c)
        If p < 0 Then
            ' wrapped provider is not in the token - use default one
            realToken = providerManifestToken
            providerInvariantName = Me.DefaultWrappedProviderName
        Else
            ' extract provider name from the token
            providerInvariantName = providerManifestToken.Substring(0, p)
            realToken = providerManifestToken.Substring(p + 1)
        End If

        ' retrieve wrapped provider manifest
        Dim services As DbProviderServices = GetProviderServicesByName(providerInvariantName)

        Return services.CreateDatabaseScript(realToken, storeItemCollection)
    End Function

    ''' <summary>
    ''' Gets provider services object given provider invariant name.
    ''' </summary>
    ''' <param name="providerInvariantName">Provider invariant name.</param>
    ''' <returns><see cref="DbProviderServices"/> object for a given invariant name.</returns>
    Private Shared Function GetProviderServicesByName(ByVal providerInvariantName As String) As DbProviderServices
        Dim factory As DbProviderFactory = DbProviderFactories.GetFactory(providerInvariantName)
        If factory Is Nothing Then
            Throw New ArgumentException("Invalid provider factory: " & providerInvariantName)
        End If

        Dim serviceProvider As IServiceProvider = TryCast(factory, IServiceProvider)
        If serviceProvider Is Nothing Then
            Throw New ArgumentException("Provider does not support Entity Framework - IServiceProvider is not supported")
        End If

        Dim providerServices As DbProviderServices = CType(serviceProvider.GetService(GetType(DbProviderServices)), DbProviderServices)
        If providerServices Is Nothing Then
            Throw New ArgumentException("Provider does not support Entity Framework - DbProviderServices is not supported")
        End If

        Return providerServices
    End Function

    Private Function GetCreateDbCommandDefinitionFunction(ByVal providerInvariantName As String) As Func(Of DbProviderManifest, DbCommandTree, DbCommandDefinition)
        Dim result As Func(Of DbProviderManifest, DbCommandTree, DbCommandDefinition) = Nothing
        SyncLock Me.createDbCommandDefinitionFunctions
            If Not Me.createDbCommandDefinitionFunctions.TryGetValue(providerInvariantName, result) Then
                Dim ps As DbProviderServices = GetProviderServicesByName(providerInvariantName)
                Me.createDbCommandDefinitionFunctions(providerInvariantName) = AddressOf ps.CreateCommandDefinition

                result = AddressOf ps.CreateCommandDefinition
            End If
        End SyncLock

        Return result
    End Function
End Class