﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Data.Common
Imports System.Data.EntityClient
Imports System.Data.Mapping
Imports System.Data.Metadata.Edm
Imports System.IO
Imports System.Linq
Imports System.Reflection
Imports System.Text.RegularExpressions
Imports System.Web
Imports System.Xml
Imports System.Xml.Linq

''' <summary>
''' Utilities for creating <see cref="EntityConnection"/> with wrapped providers.
''' </summary>
Public NotInheritable Class EntityConnectionWrapperUtils
    Private Shared metadataWorkspaceMemoizer As New Dictionary(Of String, MetadataWorkspace)()
    Private Shared systemPublicKeyToken() As Byte = {&HB0, &H3F, &H5F, &H7F, &H11, &HD5, &HA, &H3A}
    Private Shared resRegex As New Regex("^res://(?<assembly>.*)/(?<resource>.*)$")
    Private Shared httpContextTypeName As String = "System.Web.HttpContext, System.Web, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"

    ''' <summary>
    ''' Creates the entity connection with wrappers.
    ''' </summary>
    ''' The original entity connection string.
    ''' List for wrapper providers.
    ''' EntityConnection object which is based on a chain of providers.
    Private Sub New()
    End Sub
    Public Shared Function CreateEntityConnectionWithWrappers(ByVal entityConnectionString As String, ByVal ParamArray wrapperProviders() As String) As EntityConnection
        Dim ecsb As New EntityConnectionStringBuilder(entityConnectionString)

        ' if connection string is name=EntryName, look up entry in the config file and parse it
        If Not String.IsNullOrEmpty(ecsb.Name) Then
            Dim connStr = System.Configuration.ConfigurationManager.ConnectionStrings(ecsb.Name)
            If connStr Is Nothing Then
                Throw New ArgumentException("Specified named connection string '" & ecsb.Name & "' was not found in the configuration file.")
            End If

            ecsb.ConnectionString = connStr.ConnectionString
        End If

        Dim workspace As New MetadataWorkspace
        If Not metadataWorkspaceMemoizer.TryGetValue(ecsb.ConnectionString, workspace) Then
            workspace = CreateWrappedMetadataWorkspace(ecsb.Metadata, wrapperProviders)
            metadataWorkspaceMemoizer.Add(ecsb.ConnectionString, workspace)
        End If

        Dim storeConnection = DbProviderFactories.GetFactory(ecsb.Provider).CreateConnection()
        storeConnection.ConnectionString = ecsb.ProviderConnectionString
        Dim newEntityConnection = New EntityConnection(workspace, DbConnectionWrapper.WrapConnection(storeConnection, wrapperProviders))
        Return newEntityConnection
    End Function

    Private Shared Function CreateWrappedMetadataWorkspace(ByVal metadata As String, ByVal wrapperProviderNames As IEnumerable(Of String)) As MetadataWorkspace
        Dim workspace As New MetadataWorkspace()

        ' parse Metadata keyword and load CSDL,SSDL,MSL files into XElement structures...
        Dim csdl = New List(Of XElement)
        Dim ssdl = New List(Of XElement)
        Dim msl = New List(Of XElement)
        ParseMetadata(metadata, csdl, ssdl, msl)

        ' fix all SSDL files by changing 'Provider' to our provider and modifying
        For Each ssdlFile In ssdl
            For Each providerName In wrapperProviderNames
                ssdlFile.Attribute("ProviderManifestToken").Value = ssdl(0).Attribute("Provider").Value & ";" & ssdlFile.Attribute("ProviderManifestToken").Value
                ssdlFile.Attribute("Provider").Value = providerName
            Next providerName
        Next ssdlFile

        ' load item collections from XML readers created from XElements...
        Dim eic As New EdmItemCollection(csdl.Select(Function(c) c.CreateReader()))
        Dim sic As New StoreItemCollection(ssdl.Select(Function(c) c.CreateReader()))
        Dim smic As New StorageMappingItemCollection(eic, sic, msl.Select(Function(c) c.CreateReader()))

        ' and create metadata workspace based on them.
        workspace = New MetadataWorkspace()
        workspace.RegisterItemCollection(eic)
        workspace.RegisterItemCollection(sic)
        workspace.RegisterItemCollection(smic)
        Return workspace
    End Function

    Private Shared Sub ParseMetadata(ByVal metadata As String, ByVal csdl As List(Of XElement), ByVal ssdl As List(Of XElement), ByVal msl As List(Of XElement))
        For Each component In metadata.Split(New Char() {"|"c}, StringSplitOptions.RemoveEmptyEntries).Select(Function(c) c.Trim())
            Dim translatedComponent As String = component

            If translatedComponent.StartsWith("~", StringComparison.Ordinal) Then
                Dim httpContextType = Type.GetType(httpContextTypeName, True)
                Dim context As Object = httpContextType.GetProperty("Current").GetValue(Nothing, Nothing)
                If context Is Nothing Then
                    Throw New NotSupportedException("Paths prefixed with '~' are not supported outside of ASP.NET.")
                End If

                translatedComponent = context.Server.MapPath(translatedComponent)
            End If

            If translatedComponent.StartsWith("res://", StringComparison.Ordinal) Then
                ParseResources(translatedComponent, csdl, ssdl, msl)
            ElseIf Directory.Exists(translatedComponent) Then
                ParseDirectory(translatedComponent, csdl, ssdl, msl)
            ElseIf translatedComponent.EndsWith(".csdl", StringComparison.OrdinalIgnoreCase) Then
                csdl.Add(XElement.Load(translatedComponent))
            ElseIf translatedComponent.EndsWith(".ssdl", StringComparison.OrdinalIgnoreCase) Then
                ssdl.Add(XElement.Load(translatedComponent))
            ElseIf translatedComponent.EndsWith(".msl", StringComparison.OrdinalIgnoreCase) Then
                msl.Add(XElement.Load(translatedComponent))
            Else
                Throw New NotSupportedException("Unknown metadata component: " & component)
            End If
        Next component
    End Sub

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes", Justification:="We want to ignore exceptions during loading of references.")>
    Private Shared Sub ParseResources(ByVal resPath As String, ByVal csdl As List(Of XElement), ByVal ssdl As List(Of XElement), ByVal msl As List(Of XElement))
        Dim match As Match = resRegex.Match(resPath)
        If Not match.Success Then
            Throw New NotSupportedException("Not supported resource path: " & resPath)
        End If

        Dim assemblyName As String = match.Groups("assembly").Value
        Dim resourceName As String = match.Groups("resource").Value

        Dim assembliesToConsider As New List(Of System.Reflection.Assembly)()
        If assemblyName = "*" Then
            assembliesToConsider.AddRange(AppDomain.CurrentDomain.GetAssemblies())
        Else
            assembliesToConsider.Add(System.Reflection.Assembly.Load(New AssemblyName(assemblyName)))
        End If

        Dim domainManager = AppDomain.CurrentDomain.DomainManager
        If domainManager IsNot Nothing AndAlso domainManager.EntryAssembly IsNot Nothing Then
            For Each asmName In domainManager.EntryAssembly.GetReferencedAssemblies()
                Try
                    Dim asm = System.Reflection.Assembly.Load(asmName)
                    If Not assembliesToConsider.Contains(asm) Then
                        assembliesToConsider.Add(asm)
                    End If
                Catch
                    ' ignore errors
                End Try
            Next asmName
        End If

        For Each asm As System.Reflection.Assembly In assembliesToConsider.Where(Function(_asm) (Not IsEcmaAssembly(_asm)) AndAlso (Not IsSystemAssembly(_asm)))
            Using stream As Stream = asm.GetManifestResourceStream(resourceName)
                If stream Is Nothing Then
                    Continue For
                End If

                If resourceName.EndsWith(".csdl", StringComparison.OrdinalIgnoreCase) Then
                    csdl.Add(XElement.Load(XmlReader.Create(stream)))
                    Return
                ElseIf resourceName.EndsWith(".ssdl", StringComparison.OrdinalIgnoreCase) Then
                    ssdl.Add(XElement.Load(XmlReader.Create(stream)))
                    Return
                ElseIf resourceName.EndsWith(".msl", StringComparison.OrdinalIgnoreCase) Then
                    msl.Add(XElement.Load(XmlReader.Create(stream)))
                    Return
                End If
            End Using
        Next asm

        Throw New InvalidOperationException("Resource " & resPath & " not found.")
    End Sub

    Private Shared Function IsEcmaAssembly(ByVal asm As System.Reflection.Assembly) As Boolean
        Dim publicKey() As Byte = asm.GetName().GetPublicKey()

        ' ECMA key is special, as it is only 4 bytes long
        If publicKey IsNot Nothing AndAlso publicKey.Length = 16 AndAlso publicKey(8) = &H4 Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Shared Function IsSystemAssembly(ByVal asm As System.Reflection.Assembly) As Boolean
        Dim publicKeyToken() As Byte = asm.GetName().GetPublicKeyToken()

        If publicKeyToken IsNot Nothing AndAlso publicKeyToken.Length = systemPublicKeyToken.Length Then
            For i = 0 To systemPublicKeyToken.Length - 1
                If systemPublicKeyToken(i) <> publicKeyToken(i) Then
                    Return False
                End If
            Next i

            Return True
        Else
            Return False
        End If
    End Function

    Private Shared Sub ParseDirectory(ByVal directory As String, ByVal csdl As List(Of XElement), ByVal ssdl As List(Of XElement), ByVal msl As List(Of XElement))
        For Each file In System.IO.Directory.GetFiles(directory, "*.csdl")
            csdl.Add(XElement.Load(file))
        Next file

        For Each file In System.IO.Directory.GetFiles(directory, "*.ssdl")
            ssdl.Add(XElement.Load(file))
        Next file

        For Each file In System.IO.Directory.GetFiles(directory, "*.msl")
            msl.Add(XElement.Load(file))
        Next file
    End Sub
End Class