﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data
Imports System.Data.Common
Imports System.Diagnostics
Imports System.Threading

Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbCommand"/> which traces all commands executed.
''' </summary>
Friend Class EFTracingCommand
    Inherits DbCommandWrapper
    Private Shared globalCommandID As Integer

    ''' <summary>
    ''' Initializes a new instance of the EFTracingCommand class.
    ''' </summary>
    ''' <param name="wrappedCommand">The wrapped command.</param>
    ''' <param name="commandDefinition">The command definition.</param>
    Public Sub New(ByVal wrappedCommand As DbCommand, ByVal commandDefinition As DbCommandDefinitionWrapper)
        MyBase.New(wrappedCommand, commandDefinition)
        Me.CommandID = Interlocked.Increment(globalCommandID)
    End Sub

    ''' <summary>
    ''' Gets the unique command ID.
    ''' </summary>
    ''' The command ID.
    Private _CommandID As Integer
    Public Property CommandID() As Integer
        Get
            Return _CommandID
        End Get
        Private Set(ByVal value As Integer)
            _CommandID = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the <see cref="EFTracingConnection"/> used by this <see cref="T:System.Data.Common.DbCommand"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The connection to the data source.
    ''' </returns>
    Private Shadows ReadOnly Property Connection() As EFTracingConnection
        Get
            Return CType(MyBase.Connection, EFTracingConnection)
        End Get
    End Property

    ''' <summary>
    ''' Executes the query and returns the first column of the first row in the result set returned by the query. All other columns and rows are ignored.
    ''' </summary>
    ''' <returns>
    ''' The first column of the first row in the result set.
    ''' </returns>
    Public Overrides Function ExecuteScalar() As Object
        Dim e As New CommandExecutionEventArgs(Me, "ExecuteScalar")
        e.Status = CommandExecutionStatus.Executing
        Me.Connection.RaiseExecuting(e)
        Dim sw As New Stopwatch()
        Try
            sw.Start()
            Dim result As Object = MyBase.ExecuteScalar()
            sw.Stop()
            e.Result = result
            e.Duration = sw.Elapsed
            e.Status = CommandExecutionStatus.Finished
            Me.Connection.RaiseFinished(e)
            Return result
        Catch ex As Exception
            e.Result = ex
            e.Status = CommandExecutionStatus.Failed
            Me.Connection.RaiseFailed(e)
            Throw
        End Try
    End Function

    ''' <summary>
    ''' Executes a SQL statement against a connection object.
    ''' </summary>
    ''' <returns>The number of rows affected.</returns>
    Public Overrides Function ExecuteNonQuery() As Integer
        Dim e As New CommandExecutionEventArgs(Me, "ExecuteNonQuery")
        e.Status = CommandExecutionStatus.Executing
        Dim sw As New Stopwatch()
        Me.Connection.RaiseExecuting(e)
        Try
            sw.Start()
            Dim result As Integer = MyBase.ExecuteNonQuery()
            sw.Stop()
            e.Result = result
            e.Duration = sw.Elapsed
            e.Status = CommandExecutionStatus.Finished
            Me.Connection.RaiseFinished(e)
            Return result
        Catch ex As Exception
            e.Result = ex
            e.Status = CommandExecutionStatus.Failed
            Me.Connection.RaiseFailed(e)
            Throw
        End Try
    End Function

    ''' <summary>
    ''' Executes the command text against the connection.
    ''' </summary>
    ''' <param name="behavior">An instance of <see cref="T:System.Data.CommandBehavior"/>.</param>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbDataReader"/>.
    ''' </returns>
    Protected Overrides Function ExecuteDbDataReader(ByVal behavior As CommandBehavior) As DbDataReader
        Dim e As New CommandExecutionEventArgs(Me, "ExecuteReader")
        e.Status = CommandExecutionStatus.Executing
        Me.Connection.RaiseExecuting(e)
        Try
            Dim sw As New Stopwatch()
            sw.Start()
            Dim result As DbDataReader = MyBase.ExecuteDbDataReader(behavior)
            sw.Stop()
            e.Result = result
            e.Status = CommandExecutionStatus.Finished
            e.Duration = sw.Elapsed
            Me.Connection.RaiseFinished(e)
            Return result
        Catch ex As Exception
            e.Result = ex
            e.Status = CommandExecutionStatus.Failed
            Me.Connection.RaiseFailed(e)
            Throw
        End Try
    End Function
End Class