﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports Microsoft.VisualBasic
Imports System
Imports System.Data.Common
Imports System.IO
Imports EFProviderWrapperToolkit

''' <summary>
''' Wrapper <see cref="DbConnection"/> which traces all executed commands.
''' </summary>
Public Class EFTracingConnection
    Inherits DbConnectionWrapper
    Private Shared consoleLockObject As New Object()

    ''' <summary>
    ''' Initializes a new instance of the EFTracingConnection class.
    ''' </summary>
    Public Sub New()
        Me.AddDefaultListenersFromConfiguration()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the EFTracingConnection class.
    ''' </summary>
    ''' <param name="wrappedConnection">The wrapped connection.</param>
    Public Sub New(ByVal wrappedConnection As DbConnection)
        Me.WrappedConnection = wrappedConnection
        Me.AddDefaultListenersFromConfiguration()
    End Sub

    ''' <summary>
    ''' Occurs when database command is executing.
    ''' </summary>
    Public Event CommandExecuting As EventHandler(Of CommandExecutionEventArgs)

    ''' <summary>
    ''' Occurs when database command has finished execution.
    ''' </summary>
    Public Event CommandFinished As EventHandler(Of CommandExecutionEventArgs)

    ''' <summary>
    ''' Occurs when database command execution has failed.
    ''' </summary>
    Public Event CommandFailed As EventHandler(Of CommandExecutionEventArgs)

    ''' <summary>
    ''' Gets the <see cref="T:System.Data.Common.DbProviderFactory"/> for this <see cref="T:System.Data.Common.DbConnection"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbProviderFactory"/>.
    ''' </returns>
    Protected Overrides ReadOnly Property DbProviderFactory() As DbProviderFactory
        Get
            Return EFTracingProviderFactory.Instance
        End Get
    End Property

    ''' <summary>
    ''' Gets the name of the default wrapped provider.
    ''' </summary>
    ''' <returns>Name of the default wrapped provider.</returns>
    Protected Overrides ReadOnly Property DefaultWrappedProviderName() As String
        Get
            Return EFTracingProviderConfiguration.DefaultWrappedProvider
        End Get
    End Property

    Friend Sub RaiseExecuting(ByVal e As CommandExecutionEventArgs)
        RaiseEvent CommandExecuting(Me, e)
    End Sub

    Friend Sub RaiseFinished(ByVal e As CommandExecutionEventArgs)
        RaiseEvent CommandFinished(Me, e)
    End Sub

    Friend Sub RaiseFailed(ByVal e As CommandExecutionEventArgs)
        RaiseEvent CommandFailed(Me, e)
    End Sub

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", Justification:="Component is non-localizable")>
    Private Sub AddDefaultListenersFromConfiguration()
        If EFTracingProviderConfiguration.LogToConsole Then
            AddHandler Me.CommandExecuting, Sub(sender As Object, e As CommandExecutionEventArgs)
                                                SyncLock consoleLockObject
                                                    Dim oldColor = Console.ForegroundColor

                                                    Try
                                                        Console.ForegroundColor = ConsoleColor.White
                                                        Console.WriteLine("#{0} Running {1}:", e.CommandId, e.Method)
                                                        Console.ForegroundColor = ConsoleColor.DarkGray
                                                        Console.WriteLine(e.ToTraceString())
                                                    Finally
                                                        Console.ForegroundColor = oldColor
                                                    End Try
                                                End SyncLock
                                            End Sub

            AddHandler Me.CommandFinished, Sub(sender As Object, e As CommandExecutionEventArgs)
                                               SyncLock consoleLockObject
                                                   Dim oldColor = Console.ForegroundColor

                                                   Try
                                                       Console.ForegroundColor = ConsoleColor.Green
                                                       Console.WriteLine("#{0} Command completed in {1}", e.CommandId, e.Duration)
                                                   Finally
                                                       Console.ForegroundColor = oldColor
                                                   End Try
                                               End SyncLock
                                           End Sub
            AddHandler Me.CommandFailed, Sub(sender As Object, e As CommandExecutionEventArgs)
                                             SyncLock consoleLockObject
                                                 Console.ForegroundColor = ConsoleColor.Green
                                                 Console.WriteLine("#{0} Command failed {1}", e.CommandId, e.Result)
                                             End SyncLock
                                         End Sub
        End If

        Dim logFile As String = EFTracingProviderConfiguration.LogToFile

        If logFile IsNot Nothing Then
            AddHandler Me.CommandExecuting, Sub(sender As Object, e As CommandExecutionEventArgs) File.AppendAllText(logFile, e.ToTraceString() & vbCrLf & vbCrLf)
        End If

        Dim logAction = EFTracingProviderConfiguration.LogAction
        If logAction IsNot Nothing Then
            AddHandler Me.CommandExecuting, Sub(sender As Object, e As CommandExecutionEventArgs) logAction(e)

            AddHandler Me.CommandFinished, Sub(sender As Object, e As CommandExecutionEventArgs) logAction(e)

            AddHandler Me.CommandFailed, Sub(sender As Object, e As CommandExecutionEventArgs) logAction(e)
        End If
    End Sub
End Class